<?php

/*//. KatePKG - A GUI (GTK2) Package Manger for KateOS.
Written by Bob Majdak Jr <bob@kateos.org>.

This program is released under the GNU GPL.

See $PREFIX/bin/katepkg for details .//*/

function package_list_installable() {
	
	global $w;
	global $installabe;
	global $selected;
	global $pkgmaster;
	global $pkglist;
	
	//. check if we want to force a repo sync too.
	$arg = func_get_args();
	if($arg[0] == TRUE) {
		$w->ui->mode_install->info->append_text(sprintf("%s... ",_("Syncing with Remote Repositories")));
		package_fetch_updatelist(FALSE);
		$w->ui->mode_install->info->append_text(sprintf("%s.\n",_("Done")));
	}
	
	//. setting the base statusbar help for this tab.
	$w->ui->status_text(_("View packages you can install from the repositories."));

	//. change elements of the window to let the user know it could be busy for awhile.
	$w->update_title(sprintf("%s (%s)",_("Install New Packages"),_("checking repository")));
	$w->ui->mode_install->refresh->set_sensitive(FALSE);
	$w->ui->mode_install->info->clear_text();
	package_run_ui();
	
	//. freshen oneself.
	$w->ui->mode_install->list->view->set_model($w->ui->mode_install->list->empty);
	$w->ui->mode_install->list->store->clear();
	$installable = array();

	//. run off a nonlive copy of the package master array.
	$installmaster = $pkgmaster;
	
	//. find what we do not have.
	foreach($pkglist as $pkg => $info) {
//		preg_match("/(.+)-(.+)-(.+)-(.+)$/",$pkg,$match);
		if($installmaster[$pkg]) {
			unset($installmaster[$pkg]);
		}
	}
	foreach($installmaster as $pkg => $info) {
		$installable[] = $pkg;
	}
	
	//. get rid of the temp array.
	unset($installmaster);

	//. fill the model
	foreach($installable as $pkg) {
		$w->ui->mode_install->list->store->append(array($pkg,$pkgmaster[$pkg]['pkg']));
	}

	//. switch to the filled model.
	$w->ui->mode_install->list->view->set_model($w->ui->mode_install->list->store);
	
	//. tell the user we are done.	
	$w->ui->progress->set_fraction(1);
	$w->ui->mode_install->info->append_text(sprintf("%d %s.\n",count($installable),_("packages that can be installed from the repostories")));

	//. drop anything selected.
	$selected = FALSE;
	$w->ui->mode_install->selection->unselect_all();
	$w->ui->mode_install->install->set_sensitive(FALSE);
	
	//. only click to sort automagicly once.
	if($w->ui->mode_install->list->clicked_once == FALSE) {
		$w->ui->mode_install->list->c1->clicked();
		$w->ui->mode_install->list->clicked_once = TRUE;
	}
	
	//. change elements of the window to let the user know we are done.
	$w->ui->mode_install->list->view->set_model($w->ui->mode_install->list->store);
	$w->update_title(_("Install New Packages"));
	$w->ui->mode_install->refresh->set_sensitive(TRUE);
	package_run_ui();

}

function package_view_install() {

	global $w;
	global $installable;
	global $selected;
	global $updateserver;
	global $pkgmaster;
	
	list($null, $iter) = $w->ui->mode_install->selection->get_selected_rows();
	
	if(!$iter[0]) { $iter = array(); }

	$selected = array();
	
	foreach($iter as $row) {
		$selected[] = $w->ui->mode_install->list->store->get_value(
			$w->ui->mode_install->list->store->get_iter($row),
			0
		);
	}

	$w->ui->mode_install->info->clear_text();
	$w->ui->mode_install->info->append_text(sprintf("<b>%d %s.</b>\n",count($selected),_("packages selected")));
	
	foreach($selected as $pkg) {
		$w->ui->mode_install->info->append_text(sprintf("\t%s (%s)\n",$pkg,$updateserver[$pkgmaster[$pkg]['rid']]));
	}
	
	if(count($selected > 0)) {
		$w->ui->mode_install->install->set_sensitive(TRUE);
	} else {
		$w->ui->mode_install->install->set_sensitive(FALSE);
	}

}

function package_dialog_install($action) {

	/*//. This function handles both the downloading and installation of packages from the online
	repositories and local files with two seperate loops. Downloading of all the files happens prior
	to any installation. .//*/

	global $w;
	global $selected,$updateserver,$pkgmaster,$config;
	global $download;
	global $dlpp;
	
	$lp = $rp = $pp = FALSE;
	$errc = $dlerrc = $opiter = $sizeiter = $dliter = $locally = $pkgdump = $pkgstat = FALSE;
	$local = $localmd5 = $rsize = "";
	$opcount = count($selected);
	
	//. decide if we are doing a local install.
	if(preg_match("/^\//",$selected[0])) { $locally = TRUE; }
	
	//. turn off the main window.
	$w->ui->set_sensitive(FALSE);	
	
	//. create dialog.
	if($action == PKG_INSTALL) { $dialog = new packageMegaProgressDialog(_('Installing Packages'),'installation',_('Installing packages into the system')); }
	else if($action == PKG_UPDATE) { $dialog = new packageMegaProgressDialog(_('Updating Packages'),'upgrade',_('Updating packages on the system')); }
	
	$dialog->special = new GtkButton;

	$dialog->sigcan = $dialog->cancel->connect_simple('clicked','katepkg_download_break');
	$dialog->cancel->set_label(_('Cancel'));
	$dialog->show_all();
	
	while(Gtk::events_pending() || Gdk::events_pending()) { Gtk::main_iteration(); }

	$download = TRUE;
	
	//. order by the alphabet.
	sort($selected);
	$listcount = count($selected);

	//. begin the download loop.

//	foreach($selected as $pkg) {
//.	this foreach worked up until php 5.2.2, the documentation for foreach() explains why this quit working
//.	but it does not say when this behaviour was forced. apparently that was 5.2.2.

	for($a = 0; $a < $listcount; $a++) {
		$pkg = $selected[$a];	
	
		if(!$download) { break; }

		//. things i need reset at the start of every package download.
		$local = $localmd5 = FALSE;
		
		if($locally) {
			//. local install.
			$local = $pkg;
		} else {
			//. installing a remote package.
			$local = sprintf("%s/%s",DLROOT,basename($pkgmaster[$pkg]['rurl'][0]));
			katepkg_dprintf("save as %s\n",$local);
			
			//. notify user of tasks.
			$dialog->progress->set_text(sprintf("%s...",_('scanning')));
			$dialog->progress->set_fraction(FALSE);
			$dialog->totalprogress->set_text(sprintf("%s %s",_('downloading'),$pkg));
			$dialog->status->append_text(sprintf("<b>%s %s</b>...\n",_('downloading'),$pkg),TRUE);
			while(Gtk::events_pending() || Gdk::events_pending()) { Gtk::main_iteration(); }
			
			//. grab the file size.
			while(!$rsize = katepkg_filesize_remote($pkgmaster[$pkg]['rurl'][0])) {
				if($sizeiter == $config['download_retry']) { $sizeiter = FALSE; break; }
				katepkg_array_rotate($pkgmaster[$pkg]['rurl']); //. <-- rotating mirrors FTW LOL.	
				$dialog->progress->set_text(sprintf("%s...",_('retrying...')));
				while(Gtk::events_pending() || Gdk::events_pending()) { Gtk::main_iteration(); }	
				++$sizeiter;
			}
			
			while(Gtk::events_pending() || Gdk::events_pending()) { Gtk::main_iteration(); }		
			
			//. if we failed to get the size, move on.
			if(!$rsize) {
				++$errc; ++$dlerrc;
				$dialog->status->append_text(sprintf("\t%s %s\n",_('unable to download'),$pkg),TRUE);			
				echo("failed to get remote file size.\n");
				continue;
			}
			
			//. download the package.
			while(!katepkg_check_md5($local)) {
				if(!$download) { break; }
				if($dliter == $config['download_retry']) { $dliter = FALSE; break; }
				
				//. get the md5.
				$localmd5 = @file_get_contents($pkgmaster[$pkg]['rurl'][0].".md5");
				file_put_contents($local.".md5",$localmd5);

				
				katepkg_dprintf("downloading\n\t%s\n\t%s (%s)\n\t%s\n",$pkgmaster[$pkg]['rurl'][0],$local,$rsize,$localmd5);

				//. begin download.
				if($localmd5) {
				
					$lp = fopen($local,"w");
					$rp = fopen($pkgmaster[$pkg]['rurl'][0],"r");
					$lasttime = gettimeofday(TRUE); $lastbyte = 0;
					while($recv = fread($rp,DOWNLOAD_BITE)) {
						if(!$download) { break; }
						fwrite($lp,$recv);
						
						//. rate calculation.
						$thistime = gettimeofday(TRUE); $thisbyte = ftell($lp);
						$rate = round((($thisbyte - $lastbyte) / ($thistime - $lasttime)),1);
						
						//. only update if it has been more than a so long since the last. this should prevent the ui
						//. from lagging high speed internet... or a cpu from lagging high speed internet.
						if(($thistime - $lasttime) >= 0.5) {
							$dialog->progress->set_fraction((ftell($lp) / $rsize));
							$dialog->progress->set_text(sprintf(
								"%s / %s (%s/s)",
								katepkg_filesize(ftell($lp)),
								katepkg_filesize($rsize),
								katepkg_filesize($rate)
							));
							while(Gtk::events_pending() || Gdk::events_pending()) { Gtk::main_iteration(); }
							
							//. data shuffle for the next loop rate calculations.
							$lasttime = $thistime; $lastbyte = $thisbyte;
						}

					}
					fclose($lp); fclose($rp);
				
				
				} ++$dliter;
				
			} unset($thistime,$lasttime,$thisbyte,$lastbyte,$rate);
			
			//. if the md5 do not match move on.
			if(!katepkg_check_md5($local)) {
				++$errc; ++$dlerrc;
				$dialog->status->append_text(sprintf("\t%s %s\n",_('failed to download'),$pkg),TRUE);			
				katepkg_dprintf("failed to download file.\n");
				continue;
			}
			
			$dialog->status->append_text(sprintf("\t%s\n",_('download complete')),TRUE);

			//. check for dependencies
			if($config['track_deps']) {
				$deps = katepkg_get_deps($local);
				foreach($deps as $dep) {
					katepkg_dprintf("%s depends on %s... ",$pkg,$dep);
					if(katepkg_is_installed($dep)) {
						katepkg_dprintf("ok\n");
					} else {
						katepkg_dprintf("missing.\n");
						if(!in_array($dep,$selected)) {
							$dialog->status->append_text(sprintf("\t%s %s %s\n",$pkg,_('depends on'),$dep),TRUE);
							katepkg_dprintf("adding %s to download\n",$dep);
							$selected[] = $dep;
							$listcount++;
							++$opcount;
						}
					}
				}
			}			
			
			$dialog->totalprogress->set_fraction((++$opiter / $opcount));
			$dialog->status->append_text("\n",TRUE);
			while(Gtk::events_pending() || Gdk::events_pending()) { Gtk::main_iteration(); }
		
		}
	
	}
	
	//. format the window prior to installing.
	$dialog->totalprogress->set_fraction(FALSE);
	$dialog->cancel->set_sensitive(FALSE);
	$opiter = 0;
	
	//. really since i have not build download resuming yet, we might as well get rid of half downloaded files
	//. when the user clicks cancel in mid download.
	if(!$download) {
		if(!$locally) {
			foreach($selected as $pkg) {
				$local = sprintf("%s/%s",DLROOT,basename($pkgmaster[$pkg]['rurl'][0]));
				if(!katepkg_check_md5($local)) {
//					@unlink($local);
//					@unlink($local.".md5");
				}
			}
		}
	}
	
	//. begin the operational loop.
	foreach($selected as $pkg) {	

		//. do not install if we clicked cancel or had fatal download errors.
		if((!$download) || ($config['download_errors_fatal'] && $dlerrc > 0)) { continue; }
	
		//. things i need reset at the start of every package install.
		$pkgdump = $pp = FALSE;
		$pkgstat = TRUE;	
	
		//. generate the correct file path.
		if($locally) {
			$local = $pkg;
		} else {
			$local = sprintf("%s/%s",DLROOT,basename($pkgmaster[$pkg]['rurl'][0]));
		}
		
		//. build a nice name
		preg_match("/\/([^\/]+)-([^-]+)-([^-]+)-([^-]+).tgz$/",$local,$pkgnice);
		if($config['debug']) { print_r($pkgnice); }
		
		if($action == PKG_INSTALL) { $dialog->status->append_text(sprintf("<b>%s %s</b>\n",_('installing'),basename($pkg)),TRUE); }
		else if($action == PKG_UPDATE) { $dialog->status->append_text(sprintf("<b>%s %s</b>\n",_('updating'),basename($pkg)),TRUE); }
		
		katepkg_dprintf("\ninstalling %s\n",$local);
		
		if(!$locally && !katepkg_check_md5($local)) {
			$dialog->status->append_text(sprintf("\t%s %s, %s\n",_('unable to install'),$pkg,_('invalid checksum')),TRUE);
			katepkg_dprintf("cannot install %s, invalid checksum.\n",basename($local));
			continue;
		} 
		
		//. notify user of tasks.
		$dialog->progress->set_text(' ');
		$dialog->progress->set_fraction(TRUE);
		$dialog->totalprogress->set_text(sprintf("%s %s",_('installing'),$pkg));
		while(Gtk::events_pending() || Gdk::events_pending()) { Gtk::main_iteration(); }
		
		//. install package.
		if($config['download_without_install'] && !$locally) {
			$dialog->status->append_text(sprintf("\t%s\n",_('skipping install')),TRUE);
			katepkg_dprintf("\t-> %s\n\t --> %s/%s\n",$local,PKGROOT,basename($local));
			rename($local,sprintf("%s/%s",PKGROOT,basename($local)));
		} else {

			if(katepkg_is_installed($pkgnice[1])) {
				$pkgcmd = sprintf("export LANG=en_US; pkg -u \"%s\" --nodeps --no-colors 2>&1",$local);
			} else {
				$pkgcmd = sprintf("export LANG=en_US; pkg -i \"%s\" --nodeps --no-colors 2>&1",$local);
			}
		
			//. execute installation.
			$pp = popen($pkgcmd,"r");
			while($pkgdump = fgets($pp)) {
				if(preg_match("/^ERROR/",$pkgdump)) {
					if(preg_match("/Package already installed/i",$pkgdump)) {
						$dialog->status->append_text(sprintf("\t%s\n",_('package already installed')),TRUE);
					} else {
						$dialog->status->append_text(sprintf("\t%s\n",_('install failed')),TRUE);
					}					
					katepkg_dprintf("pkg: %s\n",$pkgdump);
					$pkgstat = FALSE;
					++$errc;
				}
			}

			//. if we could not install, move on.
			if(!$pkgstat) { continue; }
			else { $dialog->status->append_text(sprintf("\t%s\n",_('success')),TRUE); }
						
			//. move the packages if the user wants to keep them.
			if($config['keep_downloaded_packages'] && !$locally) {
				katepkg_dprintf("\t-> %s\n\t--> %s/%s\n",$local,PKGROOT,basename($local));
				rename($local,sprintf("%s/%s",PKGROOT,basename($local)));
			} else if(!$locally) {
				katepkg_dprintf("\tx> %s\n",$local);
				unlink($local);
			}

			//. remove md5 file.
			if(!$locally) {
				katepkg_dprintf("\tx> %s\n",$local.".md5");
				unlink($local.".md5");
			}

		}

		$dialog->totalprogress->set_fraction((++$opiter / $opcount));
		while(Gtk::events_pending() || Gdk::events_pending()) { Gtk::main_iteration(); }

	}
	
	
	//. configure window to reflect status.
	if($errc > 0 && $download) {
		if($action == PKG_INSTALL) { $dialog->icon->set_from_file(EXECROOT."/share/pixmaps/katepkg/installation_error.png"); }
		else if($action == PKG_UPDATE) { $dialog->icon->set_from_file(EXECROOT."/share/pixmaps/katepkg/upgrade_error.png"); }
		$dialog->totalprogress->set_text(_('done with errors'));
	} else if(!$download) {
		$dialog->totalprogress->set_text(_('user canceled operation'));
	} else {
		$dialog->totalprogress->set_text(_('all packages installed successfully'));
		package_list_installed();
		package_list_updates();

		if($action == PKG_INSTALL) { package_list_installable(); }
		else if($action == PKG_UPDATE) { package_list_updates(); }
	}
	
	$selected = array();
	$dialog->cancel->disconnect($dialog->sigcan);
	$dialog->sigcan = $dialog->cancel->connect_simple('clicked',array($dialog,'response'),Gtk::RESPONSE_CANCEL);
	$dialog->cancel->set_label(_('Close'));
	$dialog->cancel->set_sensitive(TRUE);
	while(Gtk::events_pending() || Gdk::events_pending()) { Gtk::main_iteration(); }

	//. all done, sit and wait for the user to close us.	
	$dialog->run();
	
	//. destroy dialog.
	$dialog->destroy(); $dialog = FALSE;
	
	//. give back the ui.
	$w->ui->set_sensitive(TRUE);

}

function package_dialog_install_local_browse() {

	global $w;	
	global $selected;
	global $install_browse_dir;
	
	$w->ui->set_sensitive(FALSE);
	
	//. clear the selection array.
	$selected = array();	
	
	//. build the dialog.
	$dialog = new GtkFileChooserDialog;
	$dialog->set_action(Gtk::FILE_CHOOSER_ACTION_OPEN);
	$dialog->set_transient_for($w);
	$dialog->set_position(Gtk::WIN_POS_CENTER);
	$dialog->set_modal(TRUE);
	$dialog->set_title(_("Browse For Package..."));
	if($install_browse_dir) {
		$dialog->set_current_folder($install_browse_dir);	
	}	
	
	$dialog->add_button(_('Cancel'),Gtk::RESPONSE_CANCEL);
	$dialog->add_button(_('Install'),Gtk::RESPONSE_OK);
	
	//. add a filter for validly named packages.
	$dialog->filter = new GtkFileFIlter;
	$dialog->filter->set_name(sprintf("KateOS %s",_("Installation Packages")));
	$dialog->filter->add_pattern("*-*-*-*.tgz");
	$dialog->add_filter($dialog->filter);
	
	//. add a filter for all files.
	$dialog->nullfilter = new GtkFileFIlter;
	$dialog->nullfilter->set_name(_("All Files"));
	$dialog->nullfilter->add_pattern("*");
	$dialog->add_filter($dialog->nullfilter);
	
	//. fight.
	$click = $dialog->run();
	
	//. if we clicked install.
	if($click == Gtk::RESPONSE_OK) {
		$install_browse_dir = $dialog->get_current_folder();
		$selected[0] = $dialog->get_filename();
		$dialog->destroy(); $dialog = FALSE;

		if($selected[0]) {
			package_dialog_install(PKG_INSTALL);
		}

	//. if we clicked cancel.
	} else {
		$dialog->destroy(); $dialog = FALSE;
		$w->ui->set_sensitive(TRUE);
	}	

}

function package_dialog_install_directory() {

	global $w;	
	global $selected;
	global $install_browse_dir;
	
	$w->ui->set_sensitive(FALSE);	
	
	//. clear the selection array.
	$selected = array();	
	
	//. build the dialog.
	$dialog = new GtkFileChooserDialog;
	$dialog->set_action(Gtk::FILE_CHOOSER_ACTION_SELECT_FOLDER);
	$dialog->set_transient_for($w);
	$dialog->set_position(Gtk::WIN_POS_CENTER);
	$dialog->set_modal(TRUE);
	$dialog->set_title(_("Locate Directory"));
	if($install_browse_dir) {
		$dialog->set_current_folder($install_browse_dir);	
	}	
	
	$dialog->add_button(_('Cancel'),Gtk::RESPONSE_CANCEL);
	$dialog->add_button(_('Install'),Gtk::RESPONSE_OK);
	
	//. add a filter for validly named packages.
	$dialog->filter = new GtkFileFIlter;
	$dialog->filter->set_name(sprintf("KateOS %s",_("Installation Packages")));
	$dialog->filter->add_pattern("*-*-*-*.tgz");
	$dialog->add_filter($dialog->filter);
	
	//. add a filter for all files.
	$dialog->nullfilter = new GtkFileFIlter;
	$dialog->nullfilter->set_name(_("All Files"));
	$dialog->nullfilter->add_pattern("*");
	$dialog->add_filter($dialog->nullfilter);
	
	//. fight.
	$click = $dialog->run();
	
	//. if we clicked install.
	if($click == Gtk::RESPONSE_OK) {
		$install_browse_dir = $dialog->get_current_folder();
		$selected = katepkg_find_packages($dir = $dialog->get_filename());
		$dialog->destroy(); $dialog = FALSE;

		if(count($selected) > 0) {
			package_dialog_install(PKG_INSTALL);
		} else {
			$w->ui->status_text_timeout(sprintf("%s (%s)",_("No Packages Found"),$dir));
		}

	//. if we clicked cancel.
	} else {
		$dialog->destroy(); $dialog = FALSE;
		$w->ui->set_sensitive(TRUE);
	}	

}

?>